part of example;

class ToDos {
  
  Sandbox _sandbox;
  
  DivElement _contentEl;
  int _nextToDoId = 0;
  
  UListElement _todoList;
	NodeValidatorBuilder _nodeValidator;
  
  init (Sandbox sandbox) {
		_sandbox = sandbox;
  }
  
  start(Map options) {
		_nodeValidator = new NodeValidatorBuilder.common();
		_nodeValidator
			..allowElement('li', attributes: ['data-id'])
			..allowElement('span')
			..allowElement('i');
	
    _initLocalStorage();
    
    var html = ['<div id="module-todos">',
                  '<form>',
                    '<input type="text" class="input-medium">',
                    '<button type="submit" class="btn">Add</button>',
                  '</form>',
                  '<ul>',
                  '</ul>',
                '</div>'].join('');
    
    _contentEl = new Element.html(html);
    _todoList = _contentEl.querySelector('ul');
    
    options['containerEl'].append(_contentEl);
    
    window.localStorage.keys.forEach((key) => _renderToDo(key));
    
    _setupEvents();
    
    _sandbox.mediator('navigationbar', 'filter').stream.listen((filterContent) {
      _filter(filterContent);
    });
    
    _sandbox.mediator('navigationbar', 'clearfilter').stream.listen((filterContent) {
      _todoList.querySelectorAll('li span').forEach((element) => element.parent.classes.remove('hide'));
    });
  }
  
  stop() {
    _contentEl.remove();
  }
  
  _initLocalStorage() {
    if (window.localStorage.keys.length == 0) {
      var map = {
        "1": {
          "subject": "Groceries: Banas and Apples",
          "isDone": false
        },
        "2": {
          "subject": "Taxes: take care of them",
          "isDone": false
        },
        "3": {
          "subject": "Bring out trash",
          "isDone": false
        }  
      };
			
      for (var key in map.keys) {
        window.localStorage[key] = stringify(map[key]);
        _nextToDoId++;
      }
    }
    else {
      for (var key in window.localStorage.keys) {
        var intKey = int.parse(key);
        
        if (intKey > _nextToDoId) {
          _nextToDoId = intKey;
        }
        
        _nextToDoId++;
      }
    }
  }
  
  _setupEvents() {
    var input = _contentEl.querySelector('input');
    
    input.onKeyDown.listen((event) {
      if (event.keyCode == KeyCode.ENTER) {
        event.preventDefault();
        
        _addToDo(input.value);
        input.value = '';
      }
    });
    
    _contentEl.querySelector('button[type="Submit"]').onClick.listen((event) {
      event.preventDefault();
      
      if (input.value.length > 0) {
        _addToDo(input.value);
        input.value = '';
      }
    });
    
    _todoList.onClick.listen((MouseEvent event) {
      var el = event.target;
      
      if (el.classes.contains('icon-remove')) {
        _deleteToDo(el.parent);
      }
      else if (el.classes.contains('icon-ok')) {
        _toggleToDoDone(el.parent);
      }
    });
  }
  
  _renderToDo(id) {
    var todoObject = parse(window.localStorage[id.toString()]);
		
    var html = ['<li class="record-todo ', (todoObject["isDone"] ? "done" : ""),'" data-id="', id,'">',
                  '<span>', todoObject["subject"], '</span>',
                  '<i class="icon icon-ok"></i>',
                  '<i class="icon icon-remove"></i>',
                '</li>'].join('');
    _todoList.append(new Element.html(html, validator: _nodeValidator));
  }

  _addToDo(text) {
    var todoJson = stringify({
      "subject": text,
      "isDone": false
    });
    
    window.localStorage[_nextToDoId.toString()] = todoJson;
    _renderToDo(_nextToDoId);
    
    _nextToDoId++;
  }
  
  _deleteToDo(todoLIElement) {
    window.localStorage.remove(todoLIElement.dataset["id"]);
    
    todoLIElement.remove();
  }
  
  _toggleToDoDone(LIElement todoElement) {
    var done = !todoElement.classes.contains('done'); 
    var id = todoElement.dataset["id"];
    var todoObject = parse(window.localStorage[id]);
    todoObject["isDone"] = done;
    window.localStorage[id] = stringify(todoObject);
    
    if (done) {
			todoElement.classes.add('done');
    }
    else {
			todoElement.classes.remove('done');
    }
    
  }
  
  _filter(content) {
    _todoList.querySelectorAll('li span').forEach((element) {
      if (element.innerHtml.contains(content)) {
        element.parent.classes.remove('hide');
      }
      else {
        element.parent.classes.add('hide');
      }
    });
  }
}