part of dartscale;

/**
 * __Intended for internal use__
 * 
 * Represents a channel in the messagebus of the [Mediator], 
 * hosting a Map of [StreamController] which represent the topics in this channel.
 */
class MediatorChannel {
  
	Map<String, StreamController> _topics = new Map<String, StreamController>();
  
	StreamController operator [] (String topicName) {
		StreamController topic = _topics[topicName];
    if (topic == null) {
      topic = new StreamController.broadcast(sync: true);
      _topics[topicName] = topic;
    }
    
    return topic;
  }
	
}

/**
 * Implementes a messagebus, using Darts streaming API;
 * It enables lously cuppled commuinications between all
 * different parts of the application.
 * 
 * Usually it's accessed over the [Sandbox] object and can
 * be used like this:
 * 
 *     this.sandbox.mediator('mychannel', 'mytopic').stream.listen((data) {
 *         print('Hello ${data}!');
 *     });
 *     
 *     this.sandbox.mediator('mychannel', 'mytopic').add('World');
 */
class Mediator implements Function{
  
  var _channels = new Map<String, MediatorChannel>();
  
	StreamController call(String channelName, String topicName) {
		MediatorChannel channel = _channels[channelName];
    if (channel == null) {
      channel = new MediatorChannel();
      _channels[channelName] = channel;
    }
    
    return channel[topicName];
  }
	
}