import 'package:unittest/unittest.dart';
import 'package:dartscale/dartscale.dart';

class TestModule {
  
  Sandbox sandbox;
  
	init(Sandbox sandbox) {
		this.sandbox = sandbox;
	}
  
  start(options) {}
  
  stop() {}
  
}

main () {
  test("Can register Module", () {
    var core = new Core();
    
    core.register(() => new TestModule(), 'TestModule');
    
    expect(core.registered("TestModule"), equals(true));
  });
  
  test("Can register multiple Modules", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "ModuleNameOne");
    core.register(() => new TestModule(), "ModuleNameTwo");
    
    Iterable registeredModules = core.registered();
    
    expect(registeredModules.contains("ModuleNameOne"), equals(true));
    expect(registeredModules.contains("ModuleNameTwo"), equals(true));
  });
  
  test("Can register Module with custom Name", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "NewName");
    
    expect(core.registered("NewName"), equals(true));
  });
  
  test("Can unregister Module", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), 'TestModule');
    core.unregister("TestModule");
    
    expect(core.registered("TestModule"), equals(false));
  });
  
  test("Can unregister Module with custom Name", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "NewName");
    core.unregister("NewName");
    
    expect(core.registered("NewName"), equals(false));
  });
  
  test("Can start Module", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), 'TestModule');
    core.start("TestModule");
    
    expect(core.running("TestModule"), equals(true));
  });
  
  test("Can start multiple Modules", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "ModuleNameOne");
    core.register(() => new TestModule(), "ModuleNameTwo");
    
    core.startMany(["ModuleNameOne", "ModuleNameTwo"]);
    
    expect(core.running("ModuleNameOne"), equals(true));
    expect(core.running("ModuleNameTwo"), equals(true));
  });
  
  test("Can start all Modules", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "ModuleNameOne");
    core.register(() => new TestModule(), "ModuleNameTwo");
    core.register(() => new TestModule(), "ModuleNameThree");
    
    core.startAll();
    
    expect(core.running().length, equals(3));
    expect(core.running("ModuleNameOne"), equals(true));
    expect(core.running("ModuleNameTwo"), equals(true));
    expect(core.running("ModuleNameThree"), equals(true));
  });
  
  test("Can stop Module", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), 'TestModule');
    core.start("TestModule");
    core.stop("TestModule");
    
    expect(core.running("TestModule"), equals(false));
  });
  
  test("Can stop multiple Modules", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "ModuleNameOne");
    core.register(() => new TestModule(), "ModuleNameTwo");
    
    core.startMany(["ModuleNameOne", "ModuleNameTwo"]);
    core.stopMany(["ModuleNameOne", "ModuleNameTwo"]);
    
    expect(core.running("ModuleNameOne"), equals(false));
    expect(core.running("ModuleNameTwo"), equals(false));
  });
  
  test("Can stop all Modules", () {
    Core core = new Core();
    
    core.register(() => new TestModule(), "ModuleNameOne");
    core.register(() => new TestModule(), "ModuleNameTwo");
    core.register(() => new TestModule(), "ModuleNameThree");
    
    core.startAll();
    core.stopAll();
    
    expect(core.running().length, equals(0));
    expect(core.running("ModuleNameOne"), equals(false));
    expect(core.running("ModuleNameTwo"), equals(false));
    expect(core.running("ModuleNameThree"), equals(false));
  });
}

