import 'package:unittest/unittest.dart';
import 'package:dartscale/dartscale.dart';

class TestModule {
  
  static final List<TestModule> runningInstances = new List<TestModule>();
  
  Sandbox sandbox;
  
	init (Sandbox sandbox) {
		this.sandbox = sandbox;
	}
	
  start(options) {
    TestModule.runningInstances.add(this);
  }
  
  stop() {
    TestModule.runningInstances.remove(this);
  }
}

main() {
  test("Can start/stop single Module", () {
    var core = new Core();
    
    core.register(() => new TestModule(), 'TestModule');
    core.start('TestModule');
    
    expect(TestModule.runningInstances.length, equals(1));
    core.stop('TestModule');
    expect(TestModule.runningInstances.length, equals(0));
  });
  
  test("Can start/stop multiple Modules", () {
    var core = new Core();
    
		core.register(() => new TestModule(), 'TestModule');
    core.start('TestModule');
    core.start('TestModule', id: 'TestId');
    
    expect(TestModule.runningInstances.length, equals(2));
    core.stopMany(['TestModule', 'TestId']);
    expect(TestModule.runningInstances.length, equals(0));
  });
  
  test("Can all start/stop Modules", () {
    var core = new Core();
    
		core.register(() => new TestModule(), 'TestModule');
		core.register(() => new TestModule(), 'TestModuleTwo');
    core.startAll();
    
    expect(TestModule.runningInstances.length, equals(2));
    core.stopAll();
    expect(TestModule.runningInstances.length, equals(0));
  });
}