import 'package:unittest/unittest.dart';
import 'package:dartscale/dartscale.dart';

void main () {
  test("Sandbox supports Function plugins", () {
    var mediator = new Mediator();
    var sandbox = new Sandbox(mediator);
    
		sandbox.addPlugin("testMethod", (arg1, arg2) {
      return "${arg1} ${arg2}";
    });
    
    
    var result = sandbox.testMethod("Hello", "World");
    expect(result, equals("Hello World"));
  });
  
  test("Sandbox supports Object plugins", () {
    var mediator = new Mediator();
    var sandbox = new Sandbox(mediator);
    
		sandbox.addPlugin("SP", new SPTestPlugin());
    
    var result = sandbox.SP.testMethod("Hello", "World");
    expect(result, equals("Hello World"));
  });
	
	test("Sandbox supports Map Function plugins", () {
  	var mediator = new Mediator();
  	var sandbox = new Sandbox(mediator);

		sandbox.addPlugins({
			"greetMethod": (arg1, arg2) {
  			return "${arg1} ${arg2}";
  		},
		
			"add": (x, y) {
				return x + y;
			}
		});
  
  
  	var result1 = sandbox.greetMethod("Hello", "World");
		var result2 = sandbox.add(5, 10);
		
  	expect(result1, equals("Hello World"));
		expect(result2, equals(15));
  });
  
  test("Sandbox supports Map Object plugins", () {
    var mediator = new Mediator();
    var sandbox = new Sandbox(mediator);
    
		sandbox.addPlugin("SP", new SPTestPlugin());
		
		sandbox.addPlugins({
			"SP": new SPTestPlugin(),
			"Math": new AddTestPlugin()
		});
    
    var result1 = sandbox.SP.testMethod("Hello", "World");
		var result2 = sandbox.Math.add(5, 10);
		
		expect(result1, equals("Hello World"));
		expect(result2, equals(15));
  });
  
  test("Sandbox proxies Mediator", () {
    var mediator = new Mediator();
    var sandbox = new Sandbox(mediator);
    var receivedCount = 0;
    
    sandbox.mediator("channel", "topic").stream.listen((data){
      expect(data, equals("blabla"));
      receivedCount++;
    });
    
    sandbox.mediator("channel", "topic").stream.listen((data){
      expect(data, equals("blabla"));
      receivedCount++;
    });
    
    sandbox.mediator("channel", "topic").add("blabla");
    
    expect(receivedCount, equals(2));
  });
}

class SPTestPlugin {
  String testMethod (arg1, arg2) {
    return "${arg1} ${arg2}";
  }
}

class AddTestPlugin {
  add (x, y) {
    return x + y;
  }
}